
//
// StockUpdateWatcher.cs
//
// Author:
//    Tomas Restrepo (tomas.restrepo@devdeo.com)
//
// THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//

using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Xml;


namespace Devdeo.StockQuotes.Service
{

   /// <summary>
   /// Monitors changes to the Stocks.xml file
   /// </summary>
   static class StockUpdateWatcher
   {
      private static Dictionary<string, decimal> _quotes =
         new Dictionary<string, decimal>();
      private static FileSystemWatcher _watcher;

      public static event EventHandler<StockUpdateEventArgs> Update;

      public static void Start(string file)
      {
         string path = Path.GetDirectoryName(file);
         string filename = Path.GetFileName(file);

         _watcher = new FileSystemWatcher(path, filename);
         _watcher.Changed += OnFileChanged;
         _watcher.Created += OnFileChanged;

         _watcher.EnableRaisingEvents = true;
      }

      private static void OnFileChanged(object sender, FileSystemEventArgs e)
      {
         try
         {
            System.Threading.Thread.Sleep(1000);
            DateTime now = DateTime.Now;

            XmlDocument doc = new XmlDocument();
            doc.Load(e.FullPath);
            foreach ( XmlNode node in doc.SelectNodes("/Stocks/Stock") )
            {
               string symbol = node["Symbol"].InnerText;
               decimal value = XmlConvert.ToDecimal(node["LastQuote"].InnerText);
               if ( !_quotes.ContainsKey(symbol) || _quotes[symbol] != value )
               {
                  StockUpdateEventArgs upd = 
                     new StockUpdateEventArgs(symbol, value, now);
                  NotifyClients(upd);
               }
               _quotes[symbol] = value;
            }
         } catch ( Exception ex )
         {
            Console.WriteLine("Exception while processing stocks.xml:");
            Console.WriteLine(ex.ToString());
         }
      }

      private static void NotifyClients(StockUpdateEventArgs e)
      {
         if ( Update == null )
            return;

         foreach ( EventHandler<StockUpdateEventArgs> d in Update.GetInvocationList() )
         {
            d.BeginInvoke(
               null, e,
               delegate(IAsyncResult res)
               {
                  ((EventHandler<StockUpdateEventArgs>)res.AsyncState).EndInvoke(res);
               },
               d
               );
         }
      }

   } // StockUpdateWatcher

   /// <summary>
   /// EventArgs class for a notification
   /// </summary>
   class StockUpdateEventArgs : EventArgs
   {
      private string _symbol;
      private decimal _quote;
      private DateTime _updateTime;

      public string Symbol
      {
         get { return _symbol; }
      }

      public decimal Quote
      {
         get { return _quote; }
      }

      public DateTime UpdateTime
      {
         get { return _updateTime; }
      }

      public StockUpdateEventArgs(string symbol, decimal quote, DateTime dtm)
      {
         _symbol = symbol;
         _quote = quote;
         _updateTime = dtm;
      }
      

   } // class StockUpdateEventArgs

} // namespace Devdeo.StockQuotes.Service
