
//
// StockWatchService.cs
//
// Author:
//    Tomas Restrepo (tomas.restrepo@devdeo.com)
//
// THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//

using System;
using System.Collections.Generic;
using System.ServiceModel;
using System.ServiceModel.Activation;
using System.Text;
using System.Workflow;
using System.Workflow.Activities;
using System.Workflow.ComponentModel;
using System.Workflow.Runtime;

using Devdeo.StockQuotes.StockMonitor.Service;


namespace Devdeo.StockQuotes.StockMonitor
{

   /// <summary>
   /// Interface that defines our External
   /// Data Exchange Service to use with the CallExternalMethod
   /// and HandleExternalEvent activities in WF
   /// </summary>
   [ExternalDataExchange]
	public interface IStockWatchService
	{
      event EventHandler<StockQuoteChangeEventArgs> QuoteChanged;

      void Subscribe(Guid workflowInstance, string symbol);
      void Unsubscribe(Guid workflowInstance, string symbol);

   } // interface IStockWatchService


   /// <summary>
   /// Custom ExternalDataExchange service that subscribes a workflow
   /// to the WCF StockQuoteUpdateService. When we get a quote update
   /// we fire an event that is handled by the State Machine
   /// workflow that called us.
   /// </summary>
   public class StockWatchService 
      : IStockWatchService, IDisposable, IStockQuoteUpdateServiceCallback
   {
      private IStockQuoteUpdateService _service;
      private Dictionary<Guid, StockQuoteSubscription> _subscriptions;

      public event EventHandler<StockQuoteChangeEventArgs> QuoteChanged;
      
      /// <summary>
      /// Create a new instance
      /// </summary>
      public StockWatchService()
      {
         InstanceContext ctxt = new InstanceContext(this);
         _service = new StockQuoteUpdateServiceClient(ctxt, "tcpBasedBinding");

         _subscriptions = new Dictionary<Guid, StockQuoteSubscription>();
      }

      /// <summary>
      /// Subscribe the specified workflow instance to updates
      /// for the specified stock symbol
      /// </summary>
      /// <param name="workflowInstance">Instance ID</param>
      /// <param name="symbol">Stock Symbol</param>
      public void Subscribe(Guid workflowInstance, string symbol)
      {
         lock ( _subscriptions )
         {
            StockQuoteSubscription subscription = new StockQuoteSubscription();
            subscription.Symbol = symbol;
            subscription.ClientId = workflowInstance.ToString();

            _service.Subscribe(subscription);
            _subscriptions.Add(workflowInstance, subscription);
         }
      }

      /// <summary>
      /// Remove a subscription from the specified workflow Instance
      /// </summary>
      /// <param name="workflowInstance">Instance ID</param>
      /// <param name="symbol">Stock Symbol</param>
      public void Unsubscribe(Guid workflowInstance, string symbol)
      {
         lock ( _subscriptions )
         {
            StockQuoteSubscription subscription = 
               _subscriptions[workflowInstance];
            _service.Unsubscribe(subscription);
            _subscriptions.Remove(workflowInstance);
         }
      }


      /// <summary>
      /// Release service references
      /// </summary>
      public void Dispose()
      {
         if ( _service != null )
         {
            ((IDisposable)_service).Dispose();
            _service = null;
         }
      }

      /// <summary>
      /// Called by the WCF Service when a quote changes
      /// for a stock we're subscribed to. Simply fire
      /// the event to the workflow instance
      /// </summary>
      /// <param name="update"></param>
      public void OnQuoteChanged(StockQuoteUpdate update)
      {
         if ( QuoteChanged != null )
         {
            Guid wfInstance = new Guid(update.ClientId);
            StockQuoteChangeEventArgs e = new StockQuoteChangeEventArgs(
               wfInstance, update.Symbol, 
               update.Quote
               );
            try
            {
               QuoteChanged(null, e);
            } catch ( Exception ex )
            {
               Console.WriteLine(ex.ToString());
            }
         }
      }

   } // class StockWatchService


   /// <summary>
   /// EventArgs class used to notify the workflow
   /// that the quote update has been received.
   /// </summary>
   [Serializable]
   public class StockQuoteChangeEventArgs : ExternalDataEventArgs
   {
      private string _symbol;
      private decimal _quote;

      public string Symbol
      {
         get { return _symbol; }
      }

      public decimal Quote
      {
         get { return _quote; }
      }

      public StockQuoteChangeEventArgs(Guid wfInstance, string symbol, decimal quote)
         : base(wfInstance)
      {
         _symbol = symbol;
         _quote = quote;
      }

   } // class StockQuoteChangeEventArgs

} // namespace Devdeo.StockQuotes.StockMonitor

